@@ -1,6 +1,7 @@
 import numpy as np
 import scipy.sparse as sps
 import warnings
+import time
 
 from ._RiskAnalyzer import _RiskAnalyzer
 from ._solvers import _lp_solver
@@ -13,6 +14,8 @@ class BTADAnalyzer(_RiskAnalyzer):
         * getWeights
         * getRisk
         * getPositions
+        * getRiskComp
+        * getDiversification
         * viewForntiers
         * set_rrate
         * set_mktdata
@@ -21,61 +24,68 @@ class BTADAnalyzer(_RiskAnalyzer):
     """
     def __init__(self, alpha=[0.], coef=None, mktdata=None, 
                  colname='adjusted', freq='Q', hlength=3.25, calendar=None, 
-                 rtype='Sharpe', detrended=False, method='ecos'):
+                 rtype='Sharpe', detrended=False, method='ecos', name='BTAD'):
         """
         Constructor
 
         Parameters
         ----------
-        `alpha` : list, optional
+        `alpha` : `list`, optional;
             List of BTSD thresholds. The default is `[0.]`.
-        `coef` : list, optionalList 
+        `coef` : `list`, optional;
             List of positive mixture 
             coefficients. Must have the same size as `alpha`. 
             A `None` value assumes an equal weighted risk mixture.
             The vector of coefficients will be normalized to unit.
             The default is `None`.
-        `mktdata` : `pandas.DataFrame`, optional
+        `mktdata` : `pandas.DataFrame`, optional;
             Historic daily market data for portfolio components in the format
             returned by `azapy.mktData` function. The default is `None`.
-        `colname` : str, optional
+        `colname` : `str`, optional;
             Name of the price column from mktdata used in the weights 
             calibration. The default is `'adjusted'`.
-        `freq` : str, optional
-            Rate of return horizon in number of business day. it could be 
-            'Q' for quarter or 'M' for month. The default is `'Q'`.
-        `hlength` : float, optional
+        `freq` : `str`, optional;
+            Rate of return horizon. It could be 
+            `'Q'` for quarter or `'M'` for month. The default is `'Q'`.
+        `hlength` : `float`, optional;
             History length in number of years used for calibration. A 
             fractional number will be rounded to an integer number of months.
             The default is `3.25` years.
-        `calendar` : `numpy.busdaycalendar`, optional
+        `calendar` : `numpy.busdaycalendar`, optional;
             Business days calendar. If is it `None` then the calendar will 
-            be set to NYSE business calendar.
+            be set to NYSE business calendar. 
             The default is `None`.
-        `rtype` : str, optional
-            Optimization type. Possible values \n
-                'Risk' : minimization of dispersion (risk) for 
-                targeted expected rate of return. \n
-                'Sharpe' : maximization of generalized Sharpe ratio.\n
-                'Sharpe2' : minimization of the inverse generalized Sharpe 
-                ratio.\n
-                'MinRisk' : minimum dispersion (risk) portfolio.\n
-                'InvNRisk' : optimal portfolio with the same dispersion (risk)
-                value as a benchmark portfolio 
-                (e.g. equal weigthed portfolio). \n
-                'RiskAverse' : optimal portfolio for a fixed value of risk 
-                aversion factor.
-            The default is `'Sharpe'`. 
-        `detrended` : Boolean, optional
-            Designates the rate type used in the delta-risk calculations:\n
-                `True` : detrended rate of return, i.e. r - E(r), \n
-                `False` : standard rate of return. 
-            The default is `False`.
-        `method` : str, optional
+        `rtype` : `str`, optional;
+            Optimization type. Possible values: \n
+                `'Risk'` : optimal-risk portfolio for targeted expected rate of 
+                return.\n
+                `'Sharpe'` : Sharpe-optimal portfolio - maximization solution.\n
+                `'Sharpe2'` : Sharpe-optimal portfolio - minimization solution.\n
+                `'MinRisk'` : minimum risk portfolio.\n
+                `'RiskAverse'` : optimal-risk portfolio for a fixed 
+                risk-aversion factor.\n
+                `'InvNrisk'` : optimal-risk portfolio with the same risk value 
+                as a benchmark portfolio (e.g., same as equal weighted 
+                portfolio).\n
+                `'Diverse'` : optimal-diversified portfolio for targeted
+                expected rate of return (maximum of inverse 1-D).\n
+                `'Diverse2'` : optimal-diversified portfolio for targeted
+                expected rate of return (minmum of 1-D).\n
+                `'MaxDiverse'` : maximum diversified portfolio.\n
+                `'InvNdiverse'` : optimal-diversified portfolio with the same
+                diversification factor as a benchmark portfolio 
+                (e.g., same as equal weighted portfolio).\n
+                `'InvNdrr'` : optima- diversified portfolio with the same 
+                expected rate of return as a benchmark portfolio
+                (e.g., same as equal weighted portfolio).\n
+            The defauls is `'Sharpe'`.
+        `method` : `str`, optional;
             Linear programming numerical method. 
-            Could be: 'ecos', 'highs-ds', 'highs-ipm', 'highs', 
-            'interior-point', 'glpk' and 'cvxopt'.
-            The defualt is `'ecos'`.
+            Could be: `'ecos'`, `'highs-ds'`, `'highs-ipm'`, `'highs'`, 
+            `'interior-point'`, `'glpk'` and `'cvxopt'`.
+            The default is `'ecos'`.
+        `name` : `str`, optional;
+            Object name. The default is `'BTAD'`.
 
         Returns
         -------
@@ -83,7 +93,8 @@ class BTADAnalyzer(_RiskAnalyzer):
         """
         self.detrended = detrended
         
-        super().__init__(mktdata, colname, freq, hlength, calendar, rtype)
+        super().__init__(mktdata, colname, freq, hlength, calendar, 
+                         rtype, name)
         
         self._set_method(method)
 
@@ -115,7 +126,7 @@ class BTADAnalyzer(_RiskAnalyzer):
         ----------
         `rrate` : `pandas.DataFrame`
             Portfolio components historical rates of returns. The
-            columns are: "date", "symbol1", "symbol2", etc.
+            columns are: `"date"`, `"symbol1"`, `"symbol2"`, etc.
         Returns
         -------
         None
@@ -201,12 +212,14 @@ class BTADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * ll
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         # mBTAD
@@ -281,12 +294,14 @@ class BTADAnalyzer(_RiskAnalyzer):
         b_data = [0.] * (ll + 1) + [1.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -367,12 +382,14 @@ class BTADAnalyzer(_RiskAnalyzer):
         b_data = [0.] * (ll + 1) + [1.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -452,12 +469,14 @@ class BTADAnalyzer(_RiskAnalyzer):
         b_data = [self.risk, 1.] + [0.] * ll
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         # rate of return
@@ -530,12 +549,14 @@ class BTADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * ll
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         # optimal weights
@@ -617,17 +638,19 @@ class BTADAnalyzer(_RiskAnalyzer):
         b_data = [0.] * (ll + 1) + [1.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
         # BTAD-Divers
-        self.divers = 1. - res['pcost']
+        self.diverse= 1. - res['pcost']
         # mBTAD
         self.risk =  res['pcost'] / t
         # optimal weights
@@ -641,4 +664,179 @@ class BTADAnalyzer(_RiskAnalyzer):
         # BTAD thresholds
         self.secondary_risk_comp = self.alpha.copy()
         
+        return self.ww
+    
+    
+    def _risk_inv_diversification(self, d=1):
+        # Order of variables:
+        # w <- [0 : mm]
+        # then for l <- [0:ll]
+        #   u_l <- mm + l(nn+1), 
+        #   s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
+        # t <- mm + ll * (nn + 1)
+        # in total dim = mm + ll(nn + 1) + 1
+        nn = self.nn
+        mm = self.mm
+        ll = self.ll
+        
+        # build c
+        c_data = list(-self.risk_comp) + [0.] * (ll * (nn + 1) + 1)
+        
+        # build G
+        G_icol = list(range(mm)) * (nn * ll)
+        G_irow = [k  for k in range(nn * ll) for _ in range(mm)]
+        G_data = list(np.ravel(-self.rrate)) * ll
+        for l in range(ll):
+            G_icol += [mm + ll * (nn + 1)] * nn \
+                  + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
+            G_irow += list(range(l * nn, (l + 1) * nn)) * 2
+            G_data += [self.alpha[l]] * nn + [-1.] * nn 
+            
+        G_icol += list(range(mm)) +[mm + ll * (nn + 1)]
+        G_irow += [nn * ll] * (mm + 1)
+        G_data += list(-self.muk * d) + [self.mu * d]
+            
+        G_icol += list(range(mm + ll * (nn + 1) + 1))
+        G_irow += list(range(nn * ll + 1, nn * ll + 2 + mm + ll * (nn + 1)))
+        G_data += [-1.] * (mm + ll * (nn + 1) + 1)
+        
+        G_shape = (nn * ll + 2 + mm + ll * (nn + 1), mm + ll * (nn + 1) + 1)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+        
+        # build h
+        h_data = [0.] * (ll * nn + mm + ll * (nn + 1) + 2)
+        
+        #build A
+        A_icol = list(range(mm)) + [mm + ll * (nn + 1)]
+        A_irow = [0] * (mm + 1)
+        A_data = [1.] * mm + [-1.]
+    
+        for l in range(ll):
+            A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
+            A_irow += [1 + l] * (nn + 1)
+            A_data += [-1.] + [1. / nn] * nn
+            
+        A_icol += [mm + l * (nn + 1) for l in range(ll)]
+        A_irow += [ll + 1] * ll
+        A_data += list(self.coef) 
+        
+        A_shape = (ll + 2, mm + ll * (nn + 1) + 1)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+        
+        # build b
+        b_data = [0.] * (ll + 1) + [1.]
+        
+        # calc
+        toc = time.perf_counter()
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
+ 
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
+            return np.array([np.nan] * mm)
+            
+        t = res['x'][-1]
+        # BTAD-Divers
+        self.diverse= 1. + 1. / res['pcost']
+        # mBTAD
+        self.risk =  1 / t
+        # optimal weights
+        self.ww = np.array(res['x'][:mm] / t)
+        self.ww.shape = mm
+        # rate of return
+        self.RR = np.dot(self.ww, self.muk)
+        # BTAD components
+        self.primary_risk_comp = \
+            np.array([res['x'][mm + l * (nn + 1)] / t for l in range(ll)])
+        # BTAD thresholds
+        self.secondary_risk_comp = self.alpha.copy()
+        
+        return self.ww
+
+    
+    def _rr_max_diversification(self):
+        # Order of variables:
+        # w <- [0 : mm]
+        # then for l <- [0:ll]
+        #   u_l <- mm + l(nn+1), 
+        #   s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
+        # in total dim = mm + ll(nn + 1)
+        nn = self.nn
+        mm = self.mm
+        ll = self.ll
+        
+        # build c
+        c_data = list(-self.muk) + [0.] * (ll * (nn + 1))
+        
+        # build G
+        G_icol = list(range(mm)) * (nn * ll)
+        G_irow = [k  for k in range(nn * ll) for _ in range(mm)]
+        G_data = list(np.ravel(-self.rrate)) * ll
+        for l in range(ll):
+            G_icol += \
+                   list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
+            G_irow += list(range(l * nn, (l + 1) * nn))
+            G_data += [-1.] * nn 
+            
+        G_icol += list(range(mm + ll * (nn + 1)))
+        G_irow += list(range(nn * ll, nn * ll + mm + ll * (nn + 1)))
+        G_data += [-1.] * (mm + ll * (nn + 1))
+        
+        G_shape = (nn * ll + mm + ll * (nn + 1), mm + ll * (nn + 1))
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+        
+        # build h
+        h_data = []
+        for l in range(self.ll):
+            h_data += [-self.alpha[l]] * nn
+        h_data += [0.] *(mm + ll * (nn + 1))
+        
+        #build A
+        A_icol = [mm + l * (nn + 1) for l in range(ll)] + list(range(mm))
+        A_irow = [0] * (ll + mm)
+        A_data = list(self.coef) + list((self.diverse - 1.) * self.risk_comp)
+        
+        A_icol += list(range(mm))
+        A_irow += [1] * mm
+        A_data += [1.] * mm
+        
+        for l in range(ll):
+            A_icol += range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1))
+            A_irow += [2 + l] * (nn + 1)
+            A_data += [-1.] + [1./nn] * nn
+        
+        A_shape = (2 + ll, mm + ll * (nn + 1))
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+ 
+        # build b
+        b_data = [0., 1.] + [0.] * ll
+        
+        # calc
+        toc = time.perf_counter()
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
+ 
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
+            return np.array([np.nan] * mm)
+            
+        # rate of return
+        self.RR = -res['pcost']
+        # optimal weights
+        self.ww = np.array(res['x'][:mm])
+        self.ww.shape = mm
+        # BTAD components
+        self.primary_risk_comp = \
+            np.array([res['x'][mm + l * (nn + 1)] for l in range(ll)])
+        # risk
+        self.risk = np.dot(self.primary_risk_comp, self.coef)
+        # BTAD thresholds
+        self.secondary_risk_comp = self.alpha.copy()
+        # diversification
+        self.diverse = 1 - self.risk / np.dot(self.ww, self.risk_comp)
+        
         return self.ww
\ No newline at end of file
