@@ -1,6 +1,7 @@
 import numpy as np
 import scipy.sparse as sps
 import warnings
+import time
 
 from ._RiskAnalyzer import _RiskAnalyzer
 from ._solvers import _lp_solver
@@ -13,6 +14,8 @@ class MADAnalyzer(_RiskAnalyzer):
         * getWeights
         * getRisk
         * getPositions
+        * getRiskComp
+        * getDiversification
         * viewForntiers
         * set_rrate
         * set_mktdata
@@ -20,59 +23,71 @@ class MADAnalyzer(_RiskAnalyzer):
         * set_random_seed
     """
     def __init__(self, coef=[1.], mktdata=None, colname='adjusted', freq='Q',
-                 hlength=3.25, calendar=None, rtype='Sharpe', method='ecos'):
+                 hlength=3.25, calendar=None, rtype='Sharpe', method='ecos',
+                 name='MAD'):
         """
         Constructor
 
         Parameters
         ----------
-        `coef` : list, optional
+        `coef` : `list`, optional;
             Positive non-increasing list of mixture coefficients. 
             The default is `[1.]`.
-        `mktdata` : `pandas.DataFrame`, optional
+        `mktdata` : `pandas.DataFrame`, optional;
             Historic daily market data for portfolio components in the format
             returned by `azapy.mktData` function. The default is `None`.
-        `colname` : str, optional
-            Name of the price column from mktdata used in the weights
+        `colname` : `str`, optional;
+            Name of the price column from mktdata used in the weights 
             calibration. The default is `'adjusted'`.
-        `freq` : str, optional
-            Rate of return horizon in number of business day. it could be
-            'Q' for quarter or 'M' for month. The default is `'Q'`.
-        `hlength` : float, optional
-            History length in number of years used for calibration. A
+        `freq` : `str`, optional;
+            Rate of return horizon. It could be 
+            `'Q'` for quarter or `'M'` for month. The default is `'Q'`.
+        `hlength` : `float`, optional;
+            History length in number of years used for calibration. A 
             fractional number will be rounded to an integer number of months.
             The default is `3.25` years.
-        `calendar` : `numpy.busdaycalendar`, optional
-            Business days calendar. If is it `None` then the calendar will
-            be set to NYSE business calendar.
+        `calendar` : `numpy.busdaycalendar`, optional;
+            Business days calendar. If is it `None` then the calendar will 
+            be set to NYSE business calendar. 
             The default is `None`.
-        `rtype` : str, optional
-            Optimization type. Possible values \n
-                'Risk' : minimization of dispersion (risk) measure for 
-                targeted rate of return. \n
-                'Sharpe' : maximization of generalized Sharpe ratio.\n
-                'Sharpe2' : minimization of the inverse generalized Sharpe 
-                ratio.\n
-                'MinRisk' : minimum dispersion (risk) portfolio.\n
-                'InvNrisk' : optimal portfolio with the same dispersion (risk)
-                value as a benchmark portfolio 
-                (e.g. equal weighted portfolio).\n
-                'RiskAverse' : optimal portfolio for a fixed value of 
-                risk-aversion factor.
-            The default is `'Sharpe'`.
-        `method` : str, optional
-            method : string, optional
-            Linear programming numerical method.
-            Could be:'ecos', 'highs-ds', 'highs-ipm', 'highs',
-            'interior-point', 'glpk' and 'cvxopt'.
-            The defualt is `'ecos'`.
+        `rtype` : `str`, optional;
+            Optimization type. Possible values: \n
+                `'Risk'` : optimal-risk portfolio for targeted expected rate of 
+                return.\n
+                `'Sharpe'` : Sharpe-optimal portfolio - maximization solution.\n
+                `'Sharpe2'` : Sharpe-optimal portfolio - minimization solution.\n
+                `'MinRisk'` : minimum risk portfolio.\n
+                `'RiskAverse'` : optimal-risk portfolio for a fixed 
+                risk-aversion factor.\n
+                `'InvNrisk'` : optimal-risk portfolio with the same risk value 
+                as a benchmark portfolio (e.g., same as equal weighted 
+                portfolio).\n
+                `'Diverse'` : optimal-diversified portfolio for targeted
+                expected rate of return (maximum of inverse 1-D).\n
+                `'Diverse2'` : optimal-diversified portfolio for targeted
+                expected rate of return (minmum of 1-D).\n
+                `'MaxDiverse'` : maximum diversified portfolio.\n
+                `'InvNdiverse'` : optimal-diversified portfolio with the same
+                diversification factor as a benchmark portfolio 
+                (e.g., same as equal weighted portfolio).\n
+                `'InvNdrr'` : optima- diversified portfolio with the same 
+                expected rate of return as a benchmark portfolio
+                (e.g., same as equal weighted portfolio).\n
+            The defauls is `'Sharpe'`.
+        `method` : `str`, optional;
+            Linear programming numerical method. 
+            Could be: `'ecos'`, `'highs-ds'`, `'highs-ipm'`, `'highs'`, 
+            `'interior-point'`, `'glpk'` and `'cvxopt'`.
+            The default is `'ecos'`.
+        `name` : `str`, optional;
+            Object name. The default is `'MAD'`.
 
         Returns
         -------
         The object.
-
         """
-        super().__init__(mktdata, colname, freq, hlength, calendar, rtype)
+        super().__init__(mktdata, colname, freq, hlength, calendar, 
+                         rtype, name)
 
         self._set_method(method)
 
@@ -100,7 +115,7 @@ class MADAnalyzer(_RiskAnalyzer):
 
         Parameters
         ----------
-        `ww` : list (`numpy.array` or `pandas.Series`)
+        `ww` : `list` (`numpy.array` or `pandas.Series`)
             Portfolio weights.
         `rrate` : `pandas.series`, optional
             The portfolio components historical rates of returns.
@@ -112,6 +127,9 @@ class MADAnalyzer(_RiskAnalyzer):
         float :
         The value of mMAD
         """
+        self._reset_output()
+        toc = time.perf_counter()
+        
         if rrate is not None:
             self.set_rrate(rrate)
 
@@ -125,6 +143,7 @@ class MADAnalyzer(_RiskAnalyzer):
         self._risk_calc_(prate)
         self.RR = np.dot(w, self.muk)
         self.ww = w
+        self.time_level1 = time.perf_counter() - toc
 
         return self.risk
 
@@ -147,6 +166,7 @@ class MADAnalyzer(_RiskAnalyzer):
         self.secondary_risk_comp = \
             np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
         self.risk = np.dot(self.primary_risk_comp, self.coef)
+        self.status = 0
 
         return self.risk
 
@@ -210,12 +230,14 @@ class MADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * ll
 
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         # mMAD
@@ -290,12 +312,14 @@ class MADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * (ll + 1)
 
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         t = res['x'][-1]
@@ -378,12 +402,14 @@ class MADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * (ll + 1)
 
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         t = res['x'][-1]
@@ -462,12 +488,14 @@ class MADAnalyzer(_RiskAnalyzer):
         b_data = [self.risk, 1.]  + [0.] * ll
 
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         # delta-risk values
@@ -538,12 +566,14 @@ class MADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * ll
 
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         # optimal weights
@@ -567,8 +597,8 @@ class MADAnalyzer(_RiskAnalyzer):
         # Order of variables:
         # w <- [0:mm]
         # then for l <- [0:ll]
-        # u_l <- mm + l(nn+1),
-        # s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
+        #   u_l <- mm + l(nn+1),
+        #   s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
         # and last t <- [mm + ll(nn + 1)]
         # in total dim = mm + ll(nn + 1) + 1
         ll = self.ll
@@ -611,6 +641,7 @@ class MADAnalyzer(_RiskAnalyzer):
         A_icol = list(range(mm))
         A_irow = [0] * mm
         A_data = list(self.risk_comp)
+        
         for l in range(ll):
             A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
             A_irow += [l + 1] * (nn + 1)
@@ -627,17 +658,19 @@ class MADAnalyzer(_RiskAnalyzer):
         b_data = [1.] + [0.] * (ll + 1)
     
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
     
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
     
         t = res['x'][-1]
         # mMAD-Divers
-        self.divers = 1. - res['pcost']
+        self.diverse= 1. - res['pcost']
         # mMAD
         self.risk = res['pcost'] / t
         # delta-risk values
@@ -652,4 +685,182 @@ class MADAnalyzer(_RiskAnalyzer):
         # rate of return
         self.RR = np.dot(self.ww, self.muk)
     
-        return self.ww
\ No newline at end of file
+        return self.ww
+    
+     
+    def _risk_inv_diversification(self, d=1):
+        # Order of variables:
+        # w <- [0:mm]
+        # then for l <- [0:ll]
+        #   u_l <- mm + l(nn+1),
+        #   s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
+        # and last t <- [mm + ll(nn + 1)]
+        # in total dim = mm + ll(nn + 1) + 1
+        ll = self.ll
+        nn = self.nn
+        mm = self.mm
+    
+        # build c
+        c_data = list(-self.risk_comp) + [0.] * (ll * (nn + 1) + 1)
+        
+        # build G
+        G_icol = list(range(mm)) * (nn * ll)
+        G_irow = [k  for k in range(nn * ll) for _ in range(mm)]
+        G_data = list(np.ravel(-self.rrate)) * ll
+        for l in range(ll):
+            G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
+                  + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
+            G_irow += \
+                [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)] \
+                + list(range(l * nn, (l + 1) * nn))
+            G_data += [-1.] * ((l + 1) * nn)
+            
+        G_icol += list(range(mm)) + [mm + ll * (nn + 1)]
+        G_irow += [nn * ll] * (mm + 1)
+        G_data += list(-self.muk * d) + [self.mu * d]
+            
+        G_icol += list(range(mm + ll * (nn + 1) + 1))
+        G_irow += list(range(ll * nn + 1, ll * nn + mm + ll * (nn + 1) + 2))
+        G_data += [-1.] * (mm + ll * (nn + 1) + 1)
+    
+        G_shape = (nn * ll + mm + ll * (nn + 1) + 2, mm + (nn + 1) * ll + 1)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+    
+        # build h
+        h_data = [0.] * (nn * ll + mm + ll * (nn + 1) + 2)
+    
+        # build A
+        A_icol = [mm + l * (nn + 1) for l in range(ll)]
+        A_irow = [0] * ll
+        A_data = list(self.coef)
+        
+        for l in range(ll):
+            A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
+            A_irow += [l + 1] * (nn + 1)
+            A_data += [-1.] + [1. / nn] * nn
+        
+        A_icol += list(range(mm)) + [mm + ll * (nn + 1)]
+        A_irow += [ll + 1] * (mm + 1)
+        A_data += [1.] * mm + [-1.]
+    
+        A_shape = (ll + 2, mm + (nn + 1) * ll + 1)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+    
+        # build b
+        b_data = [1.] + [0.] * (ll + 1)
+        
+        # calc
+        toc = time.perf_counter()
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
+    
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
+            return np.array([np.nan] * mm)
+    
+        t = res['x'][-1]
+        # mMAD-Divers
+        self.diverse= 1. + 1. / res['pcost']
+        # mMAD
+        self.risk = 1. / t
+        # delta-risk values
+        self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] / t \
+                                           for l in range(ll)])
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
+        # optimal weights
+        self.ww = np.array(res['x'][:mm]) / t
+        self.ww.shape = mm
+        # rate of return
+        self.RR = np.dot(self.ww, self.muk)
+    
+        return self.ww
+    
+    
+    def _rr_max_diversification(self):
+        # Order of variables:
+        # w <- [0:mm]
+        # then for l <- [0:ll]
+        #   u_l <- mm + l(nn+1),
+        #   s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
+        # in total dim = mm + ll(nn + 1)
+        nn = self.nn
+        mm = self.mm
+        ll = self.ll
+
+        # build c
+        c_data = list(-self.muk) + [0.] * (ll * (nn + 1))
+
+        # build G
+        G_icol = list(range(mm)) * (nn * ll)
+        G_irow = [k  for k in range(nn * ll) for _ in range(mm)]
+        G_data = list(np.ravel(-self.rrate)) * ll
+        for l in range(ll):
+            G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
+                  + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
+            G_irow += \
+                  [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)] \
+                  + list(range(l * nn, (l + 1) * nn))
+            G_data += [-1.] * ((l + 1) * nn)
+            
+        G_icol += list(range(mm + ll * (nn + 1)))
+        G_irow += list(range(ll * nn, ll * nn + mm + ll * (nn + 1)))
+        G_data += [-1.] * (mm + ll * (nn + 1))
+
+        G_shape = (nn * ll + mm + ll * (nn + 1), mm + (nn + 1) * ll)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+
+        # build h
+        h_data = [0.] * (nn * ll + mm + ll * (nn + 1))
+
+        # build A
+        A_icol = [mm + l * (nn + 1) for l in range(ll)] + list(range(mm))   
+        A_irow = [0] * (ll + mm)
+        A_data = list(self.coef) + list((self.diverse- 1) * self.risk_comp)  
+        
+        A_icol += list(range(mm))
+        A_irow += [1] * mm
+        A_data += [1.] * mm
+        for l in range(ll):
+            A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
+            A_irow += [l + 2] * (nn + 1)
+            A_data += [-1.] + [1. / nn] * nn
+
+        A_shape = (ll + 2, mm + (nn + 1) * ll)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+
+        # build b
+        b_data = [0., 1.]  + [0.] * ll
+
+        # calc
+        toc = time.perf_counter()
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
+
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
+            return np.array([np.nan] * mm)
+
+        # delta-risk values
+        self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] \
+                                           for l in range(ll)])
+        # risk
+        self.risk = np.dot(self.primary_risk_comp, self.coef)
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
+        # optimal weights
+        self.ww = np.array(res['x'][:mm])
+        self.ww.shape = mm
+        # rate of return
+        self.RR = -res['pcost']
+        # diversification
+        self.diverse= 1 - self.risk / np.dot(self.ww, self.risk_comp)
+
+        return self.ww
+    
\ No newline at end of file
